local L = AceLibrary("AceLocale-2.2"):new("Comments")


L:RegisterTranslations("enUS", function() return {
	["Comments"] = true,
    ["Description"] = "wowhead.com comments on quest details.",
    
    ["Minimum width"] = true,
    ["Set the minimum width of the comments frame"] = true,
    
    ["Maximum height"] = true,
    ["Set the maximum Height of the comments frame"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Comments"] = "설명",
    ["Description"] = "wowhead.com의 퀘스트 상세 설명을 표시합니다.",
} end)

local moduleName = "Comments"
local Comments = nQuestLog:NewModule(moduleName,"AceEvent-2.0","AceHook-2.1")

local tablet = AceLibrary("Tablet-2.0")
local quixote = AceLibrary("Quixote-1.0")


function Comments:OnModuleLoad()
	self.menuName = L["Comments"]
	self.description = L["Description"]
	
	nQuestLog:RegisterDefaults(moduleName, "profile", {
		["enabled"] = true,
		
		data = {
			fontSizePercent=1.0, detached=true, 
		},

		["minWidth"] = 300,
		["maxHeight"] = 400,
	})
end

function Comments:GetMenu(oldMenu)
	local menu = {
		type="group",
		name=self.menuName,
		order=oldMenu.order,
		desc=self.description,
		args = {
			toggleEnabled = oldMenu,
			
			minWidth = {
				order = oldMenu.order + 1,
				type = "range",
				name = L["Minimum width"],
				desc = L["Set the minimum width of the comments frame"],
				get = function()
					return self.db.profile.minWidth
				end,
				set = function(v)
					self.db.profile.minWidth = v
					tablet:Close('nQuestLog_Comments')
					tablet:Unregister('nQuestLog_Comments')
					self:Update()
				end,
				min=50,
				max=500,
				step=1,
			},
			maxHeight = {
				order = oldMenu.order + 2,
				type = "range",
				name = L["Maximum height"],
				desc = L["Set the maximum Height of the comments frame"],
				get = function()
					return self.db.profile.maxHeight
				end,
				set = function(v)
					self.db.profile.maxHeight = v
					tablet:Close('nQuestLog_Comments')
					tablet:Unregister('nQuestLog_Comments')
					self:Update()
				end,
				min=50,
				max=500,
				step=1,
			},
		}
	}
	return menu
end

function Comments:ToggleEnabled(v)
	self.db.profile.enabled = v
	if (v) then
		self:SecureHook(nQuestLog, "ShowLog")
		if nQuestLog:IsModuleActive('DetailsFrame') then
			self:SecureHook(nQuestLog:GetModule('DetailsFrame'), "UpdateQuestDetail")
		end
	else
		self:Unhook(nQuestLog, "ShowLog")
		if nQuestLog:IsModuleActive('DetailsFrame') then
			self:Unhook(nQuestLog:GetModule('DetailsFrame'), "UpdateQuestDetail")
		end

	end
end

function Comments:ShowLog(object, questid)
	--self.questid = questid
	if questid and LightHeaded and LightHeaded.IterateQuestComments and nQuestLog:IsModuleEnabled("DetailsFrame") then
		self:ScheduleEvent(self.Update, 0, self) --Schedule an event because tablet bugs out otherwise.
	end
end

function Comments:UpdateQuestDetail(object)
	if (LightHeaded and LightHeaded.IterateQuestComments) then
		self:Update()
	end
end


function Comments:GetQuestId()
	local questid
	if nQuestLog:IsModuleEnabled("DetailsFrame") then
		questid =  nQuestLog:GetModule("DetailsFrame").questid
	else
		questid = 1
	end
	
	return questid
end

function Comments:Update()
	-- This is pretty much a copy&paste of Comments.lua in FuBar_QuestsFu, made by kemayo.
	if not tablet:IsRegistered('nQuestLog_Comments') then
		local function urlClick(url)                                                                                                                                                                                                                                                                                                                                                                                                                                                         
			ChatFrameEditBox:Show()                                                                                                                                                                                                                                                                                                                                                                                                                                                            
			ChatFrameEditBox:SetText(url)                                                                                                                                                                                                                                                                                                                                                                                                                                                      
			ChatFrameEditBox:SetFocus()                                                                                                                                                                                                                                                                                                                                                                                                                                                        
		end                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  
		local function npcClick(npcid)                                                                                                                                                                                                                                                                                                                                                                                                                                                       
			if self.npcid == npcid then                                                                                                                                                                                                                                                                                                                                                                                                                                                        
				self.npcid = nil                                                                                                                                                                                                                                                                                                                                                                                                                                                                 
			else                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
				self.npcid = npcid                                                                                                                                                                                                                                                                                                                                                                                                                                                               
			end                                                                                                                                                                                                                                                                                                                                                                                                                                                                                
			self:Update()                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
		end                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  
		local function addNpcLocs(cat, id, name)                                                                                                                                                                                                                                                                                                                                                                                                                                             
			if LightHeaded:LoadNPCData(id) then                                                                                                                                                                                                                                                                                                                                                                                                                                                
				for c,z,x,y in LightHeaded:IterateNPCLocs(id) do                                                                                                                                                                                                                                                                                                                                                                                                                                 
					cat:AddLine('text', '|cFF0066FF['..x..', '..y..']|r', 'func', LightHeaded.OnHyperlinkClick, 'arg1', LightHeaded, 'arg2', 'lhref:zcoord:'..c..':'..z..':'..x..':'..y..':'..name, 'indentation', 10)                                                                                                                                                                                                                                                                             
				end                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
			else                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
				cat:AddLine('text', 'Location unknown', 'indentation', 10)                                                                                                                                                                                                                                                                                                                                                                                                                       
			end                                                                                                                                                                                                                                                                                                                                                                                                                                                                                
		end                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  
		tablet:Register('nQuestLog_Comments', 'detachedData', self.db.profile.data, 'cantAttach', true, 'dontHook', true,
			'hideWhenEmpty', true, 'showTitleWhenDetached', true, 'strata', 'HIGH', 'minWidth', self.db.profile.minWidth, 'maxHeight',self.db.profile.maxHeight,
			'children', function()
				local questid = self:GetQuestId()
				-- Much of this was taken from LightHeaded itself.
				if questid  and LightHeaded and LightHeaded.IterateQuestComments then
					local title, level, tag = quixote:GetQuestById(questid)
					local qid,sharable,_,reqlev,stype,sname,sid,etype,ename,eid,exp,rep = LightHeaded:GetQuestInfo(title, level)
					--local color = GetDifficultyColor(level)
					tablet:SetTitle("Comments")
					--tablet:SetTitleColor((color and color.r) or 1, (color and color.g) or 1, (color and color.b) or 1)

					local cat = tablet:AddCategory('text', 'LightHeaded Info', 'columns', 1, 'wrap', true, 'justify', 'CENTER')

					cat:AddLine('text', 'Quest ID: |cffffffff'..qid, 'func', urlClick, 'arg1', 'http://www.wowhead.com/?quest='..qid)
					cat:AddLine('text', 'Shareable: |cffffffff'..(shareable and 'Yes' or 'No'))
					cat:AddLine('text', 'Level: |cffffffff'..level)
					if reqlev ~= '' then cat:AddLine('text', 'Required Level: |cffffffff'..reqlev) end
					if stype ~= '' and sname ~= '' then
						if stype=='npc' then
							cat:AddLine('text', 'Starts: |cFF0066FF['..sname..']|r', 'func', npcClick, 'arg1', sid) end
						else
							cat:AddLine('text', 'Starts: |cffffffff'..sname)
						end
					if sid == self.npcid then addNpcLocs(cat, sid, sname) end
					if etype ~= '' and ename ~= '' then
						if etype == 'npc' then
							cat:AddLine('text', 'Ends: |cFF0066FF['..ename..']|r', 'func', npcClick, 'arg1', eid) end
						else
							cat:AddLine('text', 'Ends: |cffffffff'..ename)
						end
					if eid == self.npcid then addNpcLocs(cat, eid, ename) end
					if exp ~= '' then cat:AddLine('text', 'Experience: |cffffffff'..exp) end
					if rep ~= '' then
						-- Might want to do an inline category here with columns...
						cat:AddLine('text', 'Reputation:')
						for name,value in rep:gmatch("([^\031]+)\031([^\031]+)") do
							cat:AddLine('text', '|cffffffff' .. name .. ' - ' .. value, 'indentation', 10)
						end
					end
					
					cat = tablet:AddCategory('text', 'LightHeaded Comments', 'columns', 1, 'wrap', true, 'justify', 'CENTER')
					if LightHeaded:GetNumQuestComments(title, level) then
						for idx, qid, cid, rating, indent, parent, date, poster, comment in LightHeaded:IterateQuestComments(title, level) do
							cat:AddLine('indentation', indent*10, 'text', ("|cFF66FF66[%s - %s - %s]|r"):format(poster, date, rating), 'wrap', true)
							cat:AddLine('indentation', indent*10, 'text', comment, 'wrap', true)
							cat:AddLine('text', ' ') -- spacer.
						end
					else
						cat:AddLine('text', L["No comments found..."])
					end
					
					cat:AddLine('text', "Comments are all from WowHead", 'hasCheck', true, 'checked', true, 'checkIcon', 'Interface\\Addons\\LightHeaded\\images\\wh_icon', 'func', urlClick, 'arg1', 'http://www.wowhead.com')
				end
			end)
		tablet:Open('nQuestLog_Comments')
	end
	tablet:Refresh('nQuestLog_Comments')
end
